__author__ = 'ernesto'

from django.core.management.base import BaseCommand, CommandError
from optparse import make_option

from lib.css_to_less import CSSConverter

class Command(BaseCommand):

    def handle(self, *args, **options):
        self.do_run(*args)

    def do_run(self, *args):
        #TODO: convert css files
        builder = CSSConverter('../../bootstra.386')
        builder.build()



from django.db import models
import datetime


class BaseEntity(models.Model):
    """Base Entity class.

    This class is used as a base class for the models defined in the created project. This class inherits from
    the models.Model class from django. By default this class adds the created_at and updated_at columns to the model
    class that extends from this class.

    .. note::

       This class can be expanded with behaviour commonly used in a model and that hasn't been implemented on it's base
       class.
    """
    NOT_IN_FIELDS = []

    created_at = models.DateTimeField(null=True, blank=True)
    updated_at = models.DateTimeField(null=True, blank=True)

    def __init__(self, *args, **kwargs):
        now = datetime.datetime.now()
        kwargs.update(created_at=now, updated_at=now)
        models.Model.__init__(self, *args, **kwargs)

    def fields(self):
        """Method that returns a list of fields excepting the private attributes (starting with _).

        Args:
            none.

        returns:
            A list of attributes on the current model class.

        """
        return [key for key in self.__dict__.keys() if not key.startswith("_")]

    def update_fields(self):
        """Method that returns a list of fields excepting the private attributes and the ones defined on NOT_IN_FIELDS
        (by default this last list is empty).

        Args:
            none.

        returns:
            A list of attributes on the current model class excepting the private attributes and defined in
            NOT_IN_FIELDS.
        """
        return [field for field in self.fields() if field not in self.NOT_IN_FIELDS]

    class Meta:
        abstract = True


import os

from django.core.management import call_command
from django.utils.crypto import get_random_string
from .templates import render_template
import sys
from django_bootstra386view_app.utils.ansi_formater import AnsiColorsFormater
import distutils.dir_util

__dir__ = os.path.dirname(os.path.abspath(__file__))

class ProjectBuilder(object):
    def __init__(self, *args):
        """
            Setups the project_name and app_name based on *args
            *args are the parameters the user sends through the console command.

            Example:

                #in this case *args would be (project_name, )
                create_bootstra386_project [project_name]
        """

        self.args = args
        self.project_name = args[0] if self.args else None
        self.app_name = "{0}_app".format(self.project_name)
        self.type = args[1] if len(self.args) > 1 else False
        self.formater = AnsiColorsFormater()

    def build(self):
        """
            Builds the project structure and create files.
        """
        from django.core.management.base import CommandError

        self.formater.custom_message("WHITEONBLUE", '', "                                                     ")
        self.formater.custom_message("WHITEONBLUE", '', "         Django Bootstra.386 Project Creator          ")
        self.formater.custom_message("WHITEONBLUE", '', "                                                     ")

        if self.project_name is None:
            self.formater.warning_message("Please provide a project name.")
            self.formater.warning_message("Usage: create_bootstra386_project [project_name]")
            sys.exit()
        else:
            try:
                self.make_project()
                self.setup_media()
                self.setup_bootstra386()
                self.make_default_app()

                self.setup_urls()
                self.setup_settings()

                self.go_back_to_main_dir()

                self.formater.success_message("Bootstra386 Django App {0} created!".format(self.project_name.title()))
            except CommandError as e:
                self.formater.error_message("Error creating product: {0}".format(e.message))

    def make_project(self):
        """
            Creates the django project and some templates.
        """

        call_command("startproject", self.project_name)
        os.chdir(self.project_name)

        self._create_dir("templates")
        self._create_dir("templates", "index")
        self._create_file(self._get_dir("templates", "index", "index.html"), render_template("index.html"))

    def make_default_app(self):
        """
            Creates the django app, the views root folder and some example views.
        """

        self.formater.custom_message("OKCYAN", "", "Starting Django app...")

        call_command("startapp", self.app_name)

        os.remove(self._get_app_dir("views.py"))

        self._create_dir(self.app_name, "views")
        self._create_file(self._get_app_dir("views", "__init__.py"), "")
        self._create_file(self._get_app_dir("views", "index.py"), render_template("index.tm"))

        os.chdir(self.project_name)

        self.formater.custom_message("OKCYAN", "", "Done.\n")

    def setup_settings(self):
        """
            Replaces the django default settings.py for a template we provide.
        """

        self.formater.custom_message("OKCYAN", "", "Configuring settings.py...")

        # Create a random SECRET_KEY hash to put it in the main settings.
        chars = 'abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)'
        secret_key = get_random_string(50, chars)

        template_name = "settings"

        settings_file = render_template(template_name, {"app_name": self.app_name, "project_name": self.project_name,
                                                        "secret_key": secret_key})

        self._create_file("settings.py", settings_file)

        self.formater.custom_message("OKCYAN", "", "Done.\n")

    def setup_urls(self):
        """
            Replaces the django default urls.py for a template we provide.
        """

        self.formater.custom_message("OKCYAN", "", "Configuring urls.py...")

        template_name = "urls"

        urls_file = render_template(template_name, {"app_name": self.app_name})
        self._create_file("urls.py", urls_file)

        self.formater.custom_message("OKCYAN", "", "Done.\n")

    def setup_media(self):
        """
            Creates the media folder
        """

        self.formater.custom_message("OKCYAN", "", "Creating media folder...")
        mediaico_dir = os.path.join(__dir__, os.pardir, "media")

        self._create_dir("media")
        self._create_dir("media", "js")
        self._create_dir("media", "css")
        self._create_dir("media", "img")

        self.formater.custom_message("YELLOW", "", "Adding favicon...")

        distutils.dir_util.copy_tree(mediaico_dir, self._get_dir("media"))

        self.formater.custom_message("YELLOW", "", "Done.")

        self.formater.custom_message("OKCYAN", "", "Done.\n")

    def setup_bootstra386(self):
        """
        Creates the bootstra.386 directory
        """
        bootstra386_dir = os.path.join(__dir__, os.pardir, "../../bootstra.386")

        self.formater.custom_message("BOLD_MAGENTA", "", "Creating bootstra.386 folder...")
        self._create_dir("media","bootstra.386")
        self.formater.custom_message("BOLD_MAGENTA", "", "Done.\n")

        self.formater.custom_message("BOLD_MAGENTA", "", "Copying bootstra.386 files...")
        #copy all bootstrapfiles
        distutils.dir_util.copy_tree(bootstra386_dir, self._get_dir("media","bootstra.386"))

        self.formater.custom_message("BOLD_MAGENTA", "", "Done.\n")


    def go_back_to_main_dir(self):
        """
            After creating the structure this goes back to the directory where we started
            before calling os.chdir in order to create some dirs and files inside the tree.
        """

        os.chdir(os.pardir)
        os.chdir(os.pardir)

    def _create_file(self, name, content):
        """
            Created a file with [content]
        """

        with open(name, "w") as f:
            f.write(content)

    def _get_app_dir(self, *args):
        """
            Returns the app directory
        """

        return os.path.join(self.app_name, *args)

    def _get_dir(self, *args):
        """
            Returns a directory using os.path.join
        """

        return os.path.join(*args)

    def _create_dir(self, *args):
        """
            Creates a new dir joining *args
        """

        os.mkdir(self._get_dir(*args))



import sys

current_version = sys.version_info

def version_comparation(version_to_compare, operation_to_compare="<="):
    code = """ current_version {0} version_compare""".format(operation_to_compare)

    return eval(code, {"current_version": current_version, "version_compare": version_to_compare})


if __name__ == "__main__":
    print(sys.version_info)


#!/usr/bin/env python
import os
from setuptools import setup, find_packages

#define variables for setup.py
__author__ = 'Ernesto Bossi'
__version__ = "0.0.4"

__dir__ = os.path.dirname(os.path.abspath(__file__))
templates_dir = os.path.join(__dir__, "django_bootstra386view_app", "management", "commands", "templates")
templates_files = [os.path.join(templates_dir, file) for file in os.listdir(templates_dir)]

media_dir = os.path.join(__dir__, "django_bootstra386view_app", "management", "commands", "templates")
media_files = [os.path.join(media_dir, file) for file in os.listdir(media_dir)]

setup(
    name='django-classview-bootstra386',
    version=__version__,
    description='Django-Bootstra.386 generic template with class views',
    author=__author__,
    license="BSD-3",
    author_email='bossi.ernestog@gmail.com',
    url='http://bossiernesto.github.io/django-bootstra.386-classview/',
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Framework :: Django",
        "Operating System :: OS Independent",
        "Topic :: Software Development",
        "Programming Language :: Python :: 2.7",
    ],
    packages=find_packages(exclude=["django_bootstra386view"]),
    data_files=[
        (templates_dir, templates_files), (media_dir, media_files)
    ],
    include_package_data=True,
    install_requires=[
        "django_conventions",
        "south",
        "django-webtest",
        "dj_database_url",
        "simplejson",
    ],
    scripts=['bin/create_bootstra386_project'],
)




from __future__ import print_function
import datetime

class AnsiColors(object):
    HEADER = '\033[95m'
    OKCYAN = '\033[36m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    WHITEONBLUE = '\033[37;44m'
    WHITE = '\033[37m'
    YELLOW = '\033[33m'
    MAGENTA = '\033[35m'
    BOLD_MAGENTA = '\033[1;35m'

    def disable(self):
        self.HEADER = ''
        self.OKCYAN = ''
        self.OKBLUE = ''
        self.OKGREEN = ''
        self.WARNING = ''
        self.FAIL = ''
        self.ENDC = ''
        self.WHITEONBLUE = ''
        self.WHITE = ''


class AnsiColorsFormater(object):
    OK_COLORS = ["CYAN", "BLUE", "GREEN"]

    def __init__(self, ok_color="GREEN", inform_type=False, timestamp=False):
        self.restore_colors()
        self.check_ok_colors(ok_color)
        self.ok_color = ok_color.upper()
        self.decorators = []
        if inform_type:
            self.enable_type()
        if timestamp:
            self.enable_timestamp()
        self.message_timestamp = "%Y-%m-%d %H:%M:%S"


    def success_message(self, *data):
        color = self.ok_color
        return self._print(getattr(self.colorer, "OK{0}".format(color)), '', *data)

    def info_message(self, *data):
        return self._print(self.colorer.OKBLUE, '', *data)

    def warning_message(self, *data):
        return self._print(self.colorer.WARNING, '', *data)

    def error_message(self, *data):
        return self._print(self.colorer.FAIL, '', *data)

    def custom_message(self, initial_color="WHITE", end='', *data):
        try:
            initial_delimiter = getattr(self.colorer, initial_color)
        except Exception:
            raise Exception('{0} is not a valir initial color delimiter'.format(initial_color))
        return self._print(initial_delimiter, end, *data)

    def _print(self, initial, end='', *data):
        import inspect

        initial = self.decorate_initial(initial, inspect.stack()[1][3])
        new_data = (initial,) + tuple(data) + (end or self.colorer.ENDC,)
        return print(*new_data)

    def decorate_initial(self, initial, caller):
        args = {"message_timestamp": self.message_timestamp, "caller": caller}

        for decorator in self.decorators:
            initial = decorator.decorate(initial, **args)

        if len(self.decorators) > 0:
            initial += ":"
        return initial

    def check_ok_colors(self, color):
        if not color in self.OK_COLORS:
            raise Exception('{0} is not from one of the valid colors'.format(color))

    def enable_type(self):
        self.decorators.append(MessageTypeFormatter())

    def disable_type(self):
        self.remove_decorator(MessageTypeFormatter)

    def disable_timestamp(self):
        self.remove_decorator(TimeFormatter)

    def remove_decorator(self, type_decorator):
        for decorator in self.decorators:
            if isinstance(decorator, type_decorator):
                self.decorators.remove(decorator)

    def enable_timestamp(self):
        self.decorators.append(TimeFormatter())

    def disable_colors(self):
        self.colorer.disable()

    def restore_colors(self):
        self.colorer = AnsiColors()


class FormaterDecorator(object):
    def decorate(self, initial, *args, **kwargs):
        raise NotImplementedError


class TimeFormatter(FormaterDecorator):
    def decorate(self, initial, *args, **kwargs):
        initial += '[{0}]'.format(datetime.datetime.now().strftime(kwargs["message_timestamp"]))
        return initial


class MessageTypeFormatter(FormaterDecorator):
    def decorate(self, initial, *args, **kwargs):
        initial += '[{0}]'.format(kwargs["caller"].replace('_message', '').title())
        return initial


if __name__ == '__main__':
    formater = AnsiColorsFormater()
    formater.success_message('Hola esto funco bien')
    formater.warning_message('Esto es un warning')
    formater.enable_timestamp()
    formater.success_message('Hola esto funco bien')
    formater.enable_type()
    formater.success_message('Hola esto funco bien')
    formater.disable_type()
    formater.success_message('Hola esto funco bien')
    formater.custom_message("OKCYAN",'','Hola')